@@ -11,16 +11,21 @@
 //       C++ interface is currently incomplete.  The C interface is
 //       used in some places because without the strings will not get
 //       converted properly when the encoding is not the same as the
-//       internal encoding used by aspell.
+//       internal encoding used by Aspell.
 // 
 
 #include <deque>
-
 #include <ctype.h>
-
 #include "settings.h"
 
+
+#ifdef USE_LOCALE
+# include <locale.h>
+#endif
+
 #include "aspell.h"
+//FIXME if Win(dos) is different
+#include <regex.h>
 
 #include "asc_ctype.hpp"
 #include "check_funs.hpp"
@@ -41,6 +46,7 @@
 
 #include "speller_impl.hpp"
 #include "data.hpp"
+#include "directory.hpp"
 
 using namespace acommon;
 
@@ -48,6 +54,7 @@ using namespace acommon;
 
 void print_ver();
 void print_help();
+void expand_expression(Config * config);
 void config();
 
 void check(bool interactive);
@@ -61,24 +68,38 @@ void personal();
 void repl();
 void soundslike();
 
+void print_error(ParmString msg)
+{
+  fputs(_("Error: "), stderr);
+  fputs(msg, stderr);
+  fputs("\n", stderr);
+}
+
+void print_error(ParmString msg, ParmString str)
+{
+  fputs(_("Error: "), stderr);
+  fprintf(stderr, msg.str(), str.str());
+  fputs("\n", stderr);
+}
+
 #define EXIT_ON_ERR(command) \
   do{PosibErrBase pe(command);\
-  if(pe.has_err()){CERR<<"Error: "<< pe.get_err()->mesg << "\n"; exit(1);}\
+  if(pe.has_err()){print_error(pe.get_err()->mesg); exit(1);}\
   } while(false)
 #define EXIT_ON_ERR_SET(command, type, var)\
   type var;\
   do{PosibErr<type> pe(command);\
-  if(pe.has_err()){CERR<<"Error: "<< pe.get_err()->mesg << "\n"; exit(1);}\
+  if(pe.has_err()){print_error(pe.get_err()->mesg); exit(1);}\
   else {var=pe.data;}\
   } while(false)
 #define BREAK_ON_ERR(command) \
   do{PosibErrBase pe(command);\
-  if(pe.has_err()){CERR<<"Error: "<< pe.get_err()->mesg << "\n"; break;}\
+  if(pe.has_err()){print_error(pe.get_err()->mesg); break;}\
   } while(false)
 #define BREAK_ON_ERR_SET(command, type, var)\
   type var;\
   do{PosibErr<type> pe(command);\
-  if(pe.has_err()){CERR<<"Error: "<< pe.get_err()->mesg << "\n"; break;}\
+  if(pe.has_err()){print_error(pe.get_err()->mesg); break;}\
   else {var=pe.data;}\
   } while(false)
 
@@ -145,9 +166,9 @@ struct ModeAbrv {
   const char * desc;
 };
 static const ModeAbrv mode_abrvs[] = {
-  {'e', "mode=email","enter Email mode."},
-  {'H', "mode=sgml", "enter Html/Sgml mode."},
-  {'t', "mode=tex",  "enter TeX mode."},
+  {'e', "mode=email", N_("enter Email mode.")},
+  {'H', "mode=sgml",  N_("enter Html/Sgml mode.")},
+  {'t', "mode=tex",   N_("enter TeX mode.")},
 };
 
 static const ModeAbrv *  mode_abrvs_end = mode_abrvs + 3;
@@ -185,6 +206,14 @@ static const PossibleOption * find_option(const char * str) {
 
 int main (int argc, const char *argv[]) 
 {
+#ifdef USE_LOCALE
+  setlocale (LC_ALL, "");
+#endif
+#ifdef ENABLE_NLS
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+#endif
+
   EXIT_ON_ERR(options->read_in_settings());
 
   if (argc == 1) {print_help(); return 0;}
@@ -194,7 +223,7 @@ int main (int argc, const char *argv[])
   const char           * parm;
 
   //
-  // process command line options by setting the oprepreate options
+  // process command line options by setting the appropriate options
   // in "options" and/or pushing non-options onto "argv"
   //
   PossibleOption other_opt = OPTION("",'\0',0);
@@ -224,7 +253,7 @@ int main (int argc, const char *argv[])
 	while (j != mode_abrvs_end && j->abrv != argv[i][1]) ++j;
 	if (j == mode_abrvs_end) {
 	  o = find_option(argv[i][1]);
-	  if (argv[i][1] == 'v' && argv[i][2] == 'v') 
+	  if (argv[i][1] == 'v' && argv[i][2] == 'v')
 	    // Hack for -vv
 	    parm = argv[i] + 3;
 	  else
@@ -237,21 +266,20 @@ int main (int argc, const char *argv[])
 	}
       }
       if (o == possible_options_end) {
-	CERR << "Error: Invalid Option: " << argv[i] << "\n";
+	print_error(_("Invalid Option: %s"), argv[i]);
 	return 1;
       }
       if (o->num_arg == 0) {
 	if (parm[0] != '\0') {
-	  CERR << "Error: " << String(argv[i], parm - argv[i])
-	       << " does not take any parameters." << "\n";
+	  print_error(_(" does not take any parameters."), 
+		      String(argv[i], parm - argv[i]));
 	  return 1;
 	}
 	i += 1;
       } else { // o->num_arg == 1
 	if (parm[0] == '\0') {
 	  if (i + 1 == argc) {
-	    CERR << "Error: You must specify a parameter for " 
-		 << argv[i] << "\n";
+	    print_error(_("You must specify a parameter for %s"), argv[i]);
 	    return 1;
 	  }
 	  parm = argv[i + 1];
@@ -276,12 +304,12 @@ int main (int argc, const char *argv[])
   }
 
   if (args.empty()) {
-    CERR << "Error: You must specify an action" << "\n";
+    print_error(_("You must specify an action"));
     return 1;
   }
 
   //
-  // perform the requisted action
+  // perform the requested action
   //
   String action_str = args.front();
   args.pop_front();
@@ -310,13 +338,13 @@ int main (int argc, const char *argv[])
   else if (action_str == "merge")
     action = do_merge;
   else {
-    CERR << "Error: Unknown Action: " << action_str << "\n";
+    print_error(_("Unknown Action: %s"),  action_str);
     return 1;
   }
 
   if (action != do_other) {
     if (args.empty()) {
-      CERR << "Error: Unknown Action: " << action_str << "\n";
+      print_error(_("Unknown Action: %s"),  action_str);
       return 1;
     }
     String what_str = args.front();
@@ -332,8 +360,8 @@ int main (int argc, const char *argv[])
     else if (what_str == "repl")
       repl();
     else {
-      CERR << "Error: Unknown Action: " << action_str 
-	   << " " << what_str << "\n";
+      print_error(_("Unknown Action: %s"),
+		  String(action_str + " " + what_str));
       return 1;
     }
   }
@@ -356,9 +384,18 @@ void config ()
 {
   StackPtr<Config> config(new_config());
   EXIT_ON_ERR(config->read_in_settings(options));
-
-  if (args.size() == 0)
+  if ((args.size() > 0) &&
+      (args[0] == "+e")) {
+    args.pop_front();
+    if (args.size() == 0) {
+      args.push_front("all");
+    }
+    expand_expression(config);
+    args.pop_front();
+  }
+  if (args.size() == 0) {
     config->write_to_stream(COUT);
+  }
   else {
     EXIT_ON_ERR_SET(config->retrieve(args[0]), String, value);
     COUT << value << "\n";
@@ -410,7 +447,7 @@ bool get_word_pair(char * line, char * & w1, char * & w2)
 {
   w2 = strchr(line, ',');
   if (!w2) {
-    CERR << "ERROR: Invalid Input\n";
+    print_error(_("Invalid Input"));
     return false;
   }
   *w2 = '\0';
@@ -451,7 +488,7 @@ DocumentChecker * new_checker(AspellSpeller * speller,
 
 #define BREAK_ON_SPELLER_ERR\
   do {if (aspell_speller_error(speller)) {\
-    CERR<<"Error: "<< aspell_speller_error_message(speller) << "\n"; break;\
+    print_error(aspell_speller_error_message(speller)); break;\
   } } while (false)
 
 void pipe() 
@@ -470,7 +507,7 @@ void pipe()
   AspellCanHaveError * ret 
     = new_aspell_speller(reinterpret_cast<AspellConfig *>(options.get()));
   if (aspell_error(ret)) {
-    CERR << "Error: " << aspell_error_message(ret) << "\n";
+    print_error(aspell_error_message(ret));
     exit(1);
   }
   AspellSpeller * speller = to_aspell_speller(ret);
@@ -679,7 +716,7 @@ void check(bool interactive)
 
   if (interactive) {
     if (args.size() == 0) {
-      CERR << "Error: You must specify a file name.\n";
+      print_error(_("You must specify a file name."));
       exit(-1);
     }
     
@@ -689,15 +726,13 @@ void check(bool interactive)
 
     in = fopen(file_name.c_str(), "r");
     if (!in) {
-      CERR << "Error: Could not open the file \"" << file_name
-	   << "\" for reading.\n";
+      print_error(_("Could not open the file \"%s\" for reading"), file_name);
       exit(-1);
     }
     
     out = fopen(new_name.c_str(), "w");
     if (!out) {
-      CERR << "Error: Could not open the file \"" << file_name
-           << "\" for writing.  File not saved.\n";
+      print_error(_("Could not open the file \"%s\"  for writing. File not saved."), file_name);
       exit(-1);
     }
 
@@ -710,7 +745,7 @@ void check(bool interactive)
     else if (m == "ispell")
       mapping.to_ispell();
     else {
-      CERR << "Error: Invalid keymapping: " << m << "\n";
+      print_error(_("Invalid keymapping: %s"), m);
       exit(-1);
     }
 
@@ -721,7 +756,7 @@ void check(bool interactive)
   AspellCanHaveError * ret 
     = new_aspell_speller(reinterpret_cast<AspellConfig *>(options.get()));
   if (aspell_error(ret)) {
-    CERR << "Error: " << aspell_error_message(ret) << "\n";
+    print_error(aspell_error_message(ret));
     exit(1);
   }
   AspellSpeller * speller = to_aspell_speller(ret);
@@ -731,14 +766,14 @@ void check(bool interactive)
   word_choices = new Choices;
 
   menu_choices = new Choices;
-  menu_choices->push_back(Choice(mapping[Ignore],     "Ignore"));
-  menu_choices->push_back(Choice(mapping[IgnoreAll],  "Ignore all"));
-  menu_choices->push_back(Choice(mapping[Replace],    "Replace"));
-  menu_choices->push_back(Choice(mapping[ReplaceAll], "Replace all"));
-  menu_choices->push_back(Choice(mapping[Add],        "Add"));
-  menu_choices->push_back(Choice(mapping[AddLower],   "Add Lower"));
-  menu_choices->push_back(Choice(mapping[Abort],      "Abort"));
-  menu_choices->push_back(Choice(mapping[Exit],       "Exit"));
+  menu_choices->push_back(Choice(mapping[Ignore],     _("Ignore")));
+  menu_choices->push_back(Choice(mapping[IgnoreAll],  _("Ignore all")));
+  menu_choices->push_back(Choice(mapping[Replace],    _("Replace")));
+  menu_choices->push_back(Choice(mapping[ReplaceAll], _("Replace all")));
+  menu_choices->push_back(Choice(mapping[Add],        _("Add")));
+  menu_choices->push_back(Choice(mapping[AddLower],   _("Add Lower")));
+  menu_choices->push_back(Choice(mapping[Abort],      _("Abort")));
+  menu_choices->push_back(Choice(mapping[Exit],       _("Exit")));
 
   String new_word;
   Vector<String> sug_con;
@@ -819,7 +854,7 @@ void check(bool interactive)
       case Exit:
 	goto exit_loop;
       case Abort:
-	prompt("Are you sure you want to abort? ");
+	prompt(_("Are you sure you want to abort? "));
 	get_choice(choice);
 	if (choice == 'y' || choice == 'Y')
 	  goto abort_loop;
@@ -839,7 +874,7 @@ void check(bool interactive)
 	break;
       case Replace:
       case ReplaceAll:
-	prompt("With: ");
+	prompt(_("With: "));
 	get_line(new_word);
 	if (new_word.size() == 0)
 	  goto choice_prompt;
@@ -853,7 +888,7 @@ void check(bool interactive)
 	if (choice >= '1' && choice < (char)suggestions_size + '1') { 
 	  state->replace(sug_con[choice-'1']);
 	} else {
-	  error("Sorry that is an invalid choice!");
+	  error(_("Sorry that is an invalid choice!"));
 	  goto choice_loop;
 	}
       }
@@ -1064,9 +1099,9 @@ void master () {
 
   } else if (action == do_merge) {
     
-    CERR << "Can't merge a master word list yet.  Sorry\n";
+    print_error(_("Can't merge a master word list yet. Sorry."));
     exit (1);
-  
+    
   } else if (action == do_dump) {
 
     EXIT_ON_ERR_SET(add_data_set(config->retrieve("master-path"), 
@@ -1102,8 +1137,8 @@ void personal () {
 
     if (action == do_create) {
       if (file_exists(speller->config()->retrieve("personal-path"))) {
-        CERR << "Sorry I won't overwrite \"" 
-             << speller->config()->retrieve("personal-path") << "\"" << "\n";
+        print_error(_("Sorry I won't overwrite \"%s\""), 
+		    speller->config()->retrieve("personal-path"));
         exit (1);
       }
       speller->personal_word_list().data->clear();
@@ -1158,8 +1193,8 @@ void repl() {
 
     if (action == do_create) {
       if (file_exists(speller->config()->retrieve("repl-path"))) {
-        CERR << "Sorry I won't overwrite \"" 
-             << speller->config()->retrieve("repl-path") << "\"" << "\n";
+        print_error(_("Sorry I won't overwrite \"%s\""),
+		    speller->config()->retrieve("repl-path"));
         exit (1);
       }
       speller->personal_repl().clear();
@@ -1244,40 +1279,159 @@ void print_help_line(char abrv, char dont_abrv, const char * name,
     command += "=<str>";
   if (type == KeyInfoInt)
     command += "=<int>";
-  printf("  %-27s %s\n", command.c_str(), desc);
+  printf("  %-27s %s\n", command.c_str(), gettext (desc));
+}
+
+void expand_expression(Config * config){
+  StringList filtpath;
+  StringList optpath;
+  PathBrowser optionpath;
+  PathBrowser filterpath;
+  String candidate;
+  String toload;
+  size_t locate_ending=0;
+  size_t eliminate_path=0;
+  size_t hold_eliminator=0;
+//FIXME if Win(dos) is different
+  regex_t seekfor;
+  
+  if (args.size() != 0) {
+    if (args[0] == "all") {
+      args[0]=".*";
+    }
+    config->retrieve_list("filter-path",&filtpath);
+    config->retrieve_list("option-path",&optpath);
+    filterpath=filtpath;
+    optionpath=optpath;
+    if (regcomp(&seekfor,args[0].c_str(),REG_NEWLINE|REG_NOSUB)) {
+      make_err(invalid_expression,"help",args[0]);
+      return;
+    }
+    while (filterpath.expand_file_part(&seekfor,candidate)) {
+      if (((locate_ending=candidate.rfind("-filter.so")) !=
+           candidate.length() - 10)) {
+        if ((locate_ending=candidate.rfind(".flt")) !=
+            candidate.length() - 4) {
+          continue;
+        }
+        else {
+          candidate.erase(locate_ending,4);
+          eliminate_path=0;
+          while ((hold_eliminator=candidate.find('/',eliminate_path)) < 
+                 candidate.length()) {
+            eliminate_path=hold_eliminator+1;
+          }
+          toload=candidate.erase(0,eliminate_path);
+          if (regexec(&seekfor,toload.c_str(),0,NULL,0)) {
+            continue;
+          }
+        }
+      }
+      else {
+        candidate.erase(locate_ending,10);
+        eliminate_path=0;
+        while ((hold_eliminator=candidate.find('/',eliminate_path)) < 
+               candidate.length()) {
+          eliminate_path=hold_eliminator+1;
+        }
+        if (candidate.find("lib",eliminate_path) != eliminate_path) {
+          continue;
+        }
+        candidate.erase(0,eliminate_path);
+        candidate.erase(0,3);
+        locate_ending=candidate.length();
+        toload=candidate;
+        if (regexec(&seekfor,toload.c_str(),0,NULL,0)) {
+          continue;
+        }
+        candidate+="-filter.opt";
+        if (!optionpath.expand_filename(candidate)) {
+          continue;
+        }
+      }
+      config->replace("add-filter",toload.c_str());
+    }
+    regfree(&seekfor);
+  }
 }
 
 void print_help () {
-  printf(
+  char * expandedoptionname=NULL;
+  char * tempstring=NULL;
+  size_t expandedsize=0;
+
+  expand_expression(options);
+  printf(_(
     "\n"
-    "Aspell " VERSION " alpha.  Copyright 2000 by Kevin Atkinson.\n"
+    "Aspell %s alpha.  Copyright 2000 by Kevin Atkinson.\n"
     "\n"
     "Usage: aspell [options] <command>\n"
     "\n"
     "<command> is one of:\n"
-    "  -?|help          display this help message\n"
+    "  -?|help [<expr>] display this help message\n"
+    "                    and help for filters matching <expr> if installed\n"
     "  -c|check <file>  to check a file\n"
     "  -a|pipe          \"ispell -a\" compatibility mode\n"
     "  -l|list          produce a list of misspelled words from standard input\n"
-    "  [dump] config    dumps the current configuration to stdout\n"
-    "  config <key>     prints the current value of an option\n"
-    "  soundslike       returns the soundslike equivalent for each word entered\n"
+    "  [dump] config [-e <expr>]  dumps the current configuration to stdout\n"
+    "  config [+e <expr>] <key>   prints the current value of an option\n"
+    "  soundslike       returns the sounds like equivalent for each word entered\n"
     "  filter           passes standard input through filters\n"
     "  -v|version       prints a version line\n"
     "  dump|create|merge master|personal|repl [word list]\n"
     "    dumps, creates or merges a master, personal, or replacement word list.\n"
     "\n"
+    "  <expr>           regular expression matching filtername(s) or `all'\n"
+    "\n"
     "[options] is any of the following:\n"
-    "\n");
+    "\n"), VERSION);
   Enumeration<KeyInfoEnumeration> els = options->possible_elements();
   const KeyInfo * k;
   while (k = els.next(), k) {
     if (k->desc == 0) continue;
-    const PossibleOption * o = find_option(k->name);
+    if ((k->type == KeyInfoDescript) &&
+        !strncmp(k->name,"filter-",7)){
+      printf(_("\n"
+               "  %s Filter: %s\n"
+               "\tNOTE: in ambiguous case prefix following options by `filter-'\n"),
+               &(k->name)[7],k->desc);
+      if (expandedoptionname != NULL) {
+        free(expandedoptionname);
+        expandedsize=0;
+      }
+      if (!strncmp(k->name,"filter-",7)) {
+        expandedoptionname=strdup(&(k->name[7]));
+        expandedsize=strlen(k->name)-7;
+      }
+      else {
+        expandedoptionname=strdup(k->name);
+        expandedsize=strlen(k->name);
+      }
+      continue;
+    }
+    else if (k->type == KeyInfoDescript) {
+      if (expandedoptionname != NULL) {
+        free(expandedoptionname);
+        expandedsize=0;
+        expandedoptionname=NULL;
+      }
+    }
+    if ((tempstring=(char*)malloc(expandedsize+strlen(k->name)+2)) == NULL) {
+      expandedoptionname=NULL;
+      continue;
+    }
+    tempstring[0]='\0';
+    if ((strlen(k->name) < expandedsize) ||
+        (expandedsize && strncmp(k->name,expandedoptionname,expandedsize))) {
+      tempstring=strncat(tempstring,expandedoptionname,expandedsize);
+      tempstring=strncat(tempstring,"-",1);
+    }
+    tempstring=strncat(tempstring,k->name,strlen(k->name));
+    const PossibleOption * o = find_option(tempstring);
     print_help_line(o->abrv, 
 		    strncmp((o+1)->name, "dont-", 5) == 0 ? (o+1)->abrv : '\0',
-		    k->name, k->type, k->desc);
-    if (strcmp(k->name, "mode") == 0) {
+		    tempstring, k->type, k->desc);
+    if (strcmp(tempstring, "mode") == 0) {
       for (const ModeAbrv * j = mode_abrvs;
            j != mode_abrvs_end;
            ++j)
@@ -1285,6 +1439,13 @@ void print_help () {
         print_help_line(j->abrv, '\0', j->mode, KeyInfoBool, j->desc, true);
       }
     }
+    if (tempstring != NULL) {
+      free(tempstring);
+      tempstring=NULL;
+    }
+  }
+  if (expandedoptionname!=NULL) {
+    free(expandedoptionname);
   }
 
 }
